#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#      lang4dcplugin.py
#
#      Copyright 2007 Jonas Fourquier <http://jonas.tuxfamily.org>
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


import re
import sys
import getopt


def main(argv):
    global dico, listesources

    lang = maj = plugin = trad = ''
    sources = []
    dico = listesources = {}
    utf2xhtml = 0


    try:
        opts, args = getopt.getopt(argv, "hs:l:m:p:t:u", ["help","source=","lang=","maj=","plugin=","trad=",'utf2xhtml'])
    except getopt.GetoptError:
        usage()
        sys.exit(2)
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            sys.exit()
        elif opt in ("-s", "--source"):
            sources = str(arg).split(' ')
        elif opt in ("-l","--lang"):
            lang = arg
        elif opt in ("-m","--maj"):
            maj = arg
        elif opt in ("-p","--plugin"):
            plugin = arg
        elif opt in ("-t","--trad"):
            trad = arg
        elif opt in ("-u","--utf2xhtml"):
            utf2xhtml = 1


    print 'lang4dcplugin'
    if len(sources) == 0:
        usage()
        sys.exit(0)
    else:
        print ' - %s source(s)'%len(sources)
        for source in sources:
            print '   - Lecture de "%s"'%source
            nb_chaine_source = readsource(source)
    if lang == '':
        lang = source + '.lang'
    print ' - Fichier de sortie : "%s"'%lang

    if maj != '':
        print ' - Mise à jours depuis "%s"'%maj

    if plugin == '':
        plugin = 'non spécifié'
    print ' - Nom du plugin : %s'%plugin

    if trad == '':
        trad = 'non spécifié'
    print ' - Traducteur : %s'%trad


    writesource(lang,sources,maj,plugin,trad,utf2xhtml,nb_chaine_source)

    print '*** Fin ***'


def readsource(source):
    global dico, listesources
    file = open(source)
    lignes = file.readlines()
    no = nb = 0

    for ligne in lignes:
        char = str(ligne)
        no += 1
        print "                  ligne %s : %s" %(no,char[0:-1])
        while re.search('__\(\'',char):
            char = re.split('__\(\'',char,1)[1]
            (key,char) = re.split('\'\)',char,1)
            nb += 1
            if key in dico:
                'Si la chaine se trouve déjà dans le dico'
                if source in listesources[key]:
                    'et le fichier source, ajout de no de ligne'
                    dico[key] += ', %s'%str(no)
                else:
                    'mais pas ce fichier source, ajout fichier + ligne'
                    dico[key] += ', %s ligne %s'%(source,str(no))
                    listesources[key].append(source)
            else:
                'si la chaine n\'existe pas dans le dico'
                dico[key] = '%s ligne %s'%(source,str(no))

    file.close()

    return nb

def readmaj(maj):
    file = open(maj)
    lignes = file.readlines()
    nextISval = False
    dicotrad = {}

    for ligne in lignes:
        if nextISval:
            nextISval = False
            if ligne[0:-1] != '':
                trad = ligne[0:-1]
                dicotrad[chaine] = trad
        elif ligne[0] == ';':
            chaine = ligne[1:-1]
            nextISval = True

    file.close()
    return dicotrad


def utf2xhtml(ligne):
    import htmlentitydefs
    xhtml = ""
    nextIStag = textINtag = False
    try:
        ligne = unicode(ligne,'utf-8','replace')
    except:
        is_utf8 = False
    else:
        is_utf8 = True

    for c in ligne:
        c2xhtml = False
        #try:
            #c = unicode(c,'utf8','replace')
        #except:
            #pass

        if c == "<":
            "Debut d'un tag"
            nextIStag = True
        elif c == ">":
            "fin d'un tag"
            nextIStag = False
        elif nextIStag:
            "si on est dans un tag"
            if textINtag:
                if c == '"':
                    "text dans un tag (ex: balise title)"
                    textINtag = True
            else:
                if c == '"':
                    "fin du text dans une balise"
                    textINtag = False
                else:
                    "Si on se trouve dans du text : convertion"
                    c2xhtml = True
        else:
            "si on ne se trouve pas dans un tag : convertion"
            c2xhtml = True

        if c2xhtml:
            o = ord(c)

            if o in htmlentitydefs.codepoint2name:
                xhtml += '&'+htmlentitydefs.codepoint2name[o]+';'
            else:
                xhtml += c
        else:
            xhtml += c

        print xhtml


    return xhtml


def writesource(filelang,sources,maj,plugin,trad,inXhtml,nb_chaine_source):

    if maj != '':
        dico4maj = readmaj(maj)

    head = body = ""


    head += '#Fichier pour la traduction du/des fichier(s) %s du plugin %s\n'%(sources,plugin)
    head += '#  - Générateur : lang4dcplugin <http://jonas.tuxfamily.org)>\n'
    head += '#  - Traducteur(s) : %s\n'%trad


    print 'Ecriture du fichier de traduction'

    nblang = nbtrad = 0

    for chaine in dico:
        if inXhtml == 1:
            'Convertion utf2xhtml'
            if maj != '':
                'à partir d\'un fichier lang'
                if chaine in dico4maj:
                    nblang += 1
                    body += '\n#%s\n'%dico[chaine]
                    body += ';%s\n'%chaine
                    nbtrad += 1
                    body += '%s\n'%utf2xhtml(dico4maj[chaine])
            else:
                'à partir du fichier source'
                chaine_utf = utf2xhtml(chaine).encode('ascii')
                if chaine_utf != chaine:
                    nblang += 1
                    body += '\n#%s\n'%dico[chaine]
                    body += ';%s\n'%chaine
                    nbtrad += 1
                    body += '%s\n'%chaine_utf
        else:
            nblang += 1
            body += '\n#%s\n'%dico[chaine]
            body += ';%s\n'%chaine
            if maj != '':
                if chaine in dico4maj:
                    nbtrad += 1
                    body += '%s\n'%dico4maj[chaine]
                else:
                    body += '\n'
            else:
                body += '\n'

    head += '#  - Nombre de chaine à traduire : %s\n\n'%nblang

    file = open(filelang,'w')
    file.write(head+body)
    file.close()

    print ''' - Nombre de chaines trouvées dans le(s) fichier(s) source(s) : %s
 - Nombre de chaines à traduire : %s
 - Nombre de chaines traduite : %s
 - Nombre de chaines non-traduite : %s'''%(nb_chaine_source,nblang,nbtrad,nblang-nbtrad)


def usage():
    print  '''lang4dcplugin : générateur de fichier.lang pour les plugin de Dotclear 1.2.x
Usage :
dcplugin2lang.py --source=<file> [OPTIONS]
  <file>            Fichier(s) qui doit être traduit (ex: mon_plugin/index.php)

Options :
  -h, --help        Cette aide
  -s, --source      Fichier(s) source(s) qui serviront à la création du .lang. Si plusieurs fichiers spécifier les mettres entre ""
  -l, --lang        Fichier de sortie de type .lang. Si non spécifié créeer un <fichier source>.lang
  -m, --maj         Ancien fichier de traduction (pour mise à jours)
  -p, --plugin      Nom du plugin
  -t, --trad        Nom du traducteur
  -u, --utf2xhtml   Convertir un fichier lang utf-8 en xhtml (é en &eacute; ...). Si la chaine n'existe pas ou que le fichier lang n'est précisé, convertis la chaine se trouvant dans le fichier source.

Exemples :
Mise à jour d'un fichier .lang :
dcplugin2lang.py --source=<file1> --maj=<file2>
  <file1>           Fichier(s) qui doit être traduit (ex: mon_plugin/index.php)
  <file2>           Fichier de traduction (ex: mon_plugin/l10n/fr/main.lang)
Convertir un fichier de langue utf-8 en fichier "simple" (é en &eacute; ...) :
dcplugin2lang.py --source=<file1> --maj=<file2> --utf2xhtml
  <file1>, <file2>  idem 1er exemple
Créer un fichier de langue non utf-8 (é en &eacute; ...) sans fichier de lang utf-8 :
dcplugin2lang.py --source=<file1> --utf2xhtml
  <file1>           idem 1er exemple


Apropos :
 Auteur     : Jonas Fourquier
 Version    : 0.200709314 Licensed   : GNU GPL V2
 Web        : http://jonas.tuxfamily.org/wiki/dotclear/lang4dcplugin
'''


main(sys.argv[1:])
exit(2)
